var db = require('./database');
const { 
    v1: uuidv1,
    v4: uuidv4,
} = require('uuid');

var wsconnections = new Set();

// Convert Date into ISO8601 string
function dateToISO (d) {
    var pad = function (n) {
        return n < 10 ? '0' + n : n;
    };
    return d.getUTCFullYear()+'-'
        + pad(d.getUTCMonth()+1)+'-'
        + pad(d.getUTCDate())+'T'
        + pad(d.getUTCHours())+':'
        + pad(d.getUTCMinutes())+':'
        + pad(d.getUTCSeconds())+'.'
        + pad(d.getUTCMilliseconds())+'000Z'
}

// Send event notification using the websocket connection
function notifyEvent (event) {
    console.log(event);
    for (let c of wsconnections) {
        c.send(JSON.stringify(event));
    }
}

module.exports = {
    // Add event into local database and notify the CamTrace server
    addCustomEvent: function (eventType, customProperty) {
        var newEvent = {
            'eventId': uuidv4(),
            'type': eventType,
            'timestamp': dateToISO(new Date()),
        };

        if (customProperty !== undefined)
            newEvent['description'] = {'customParams': {'demoParam': customProperty}};

        db.get('events').push(newEvent).value();
        notifyEvent(newEvent);
    },

    addMotionEvent: function () {
        var newEvent = {
            'eventId': uuidv4(),
            'type': "analytics.motion",
            'timestamp': dateToISO(new Date()),
            'description': {
                'params': {
                    'state': (Math.random() < 0.333 ? false : true),
                }
            }
        };

        db.get('events').push(newEvent).value();
        notifyEvent(newEvent);
    },

    addBoundingBox: function() {
        var newEvent = {
            'type': 'actor.boundingbox',
            'description': {
                'params': {
                    'width': 320,
                    'height': 160,
                    'boxes': [{
                        'actorId': "1",
                        'actorType': "car",
                        'xmin': 40,
                        'ymin': 40,
                        'xmax': 120,
                        'ymax': 80,
                    }]
                }
            }
        };

        notifyEvent(newEvent);
    },

    labels: {
        'demo.event1': 'Demo event 1',
        'demo.event2': 'Demo event 2',
        'analytics.motion': 'Motion detection event'
    },

    descriptions: {
        'demo.event1': 'custom event 1 occured',
        'demo.event2': 'custom event 2 occured with %{/customParams/demoParam} as custom property'
    },

    params: {
        'demo' : {
            'event1': 'Event 1',
            'event2': 'Event 2'
        }
    },

    addWsConnection: function(wsconn) {
        wsconnections.add(wsconn);
        wsconn.on("close", function() {
            wsconnections.delete(wsconn);
        })
    },
    purgeEvents: function(before) {
        db.get('events').remove((room) => {
            var d1 = new Date(before);
            var d2 = new Date(room.timestamp);
            return (d1 - d2 > 0);
        }).value();
    }
}
